/*
	BFileIFF.cpp
	IFF classes implementation
	
	Leo Chan
	August 1997
*/

#include "FileIFF.h"

IFFP BFileIFF :: ReadFORM(IFFChunk *form)
{
	IFFChunk subchunk;
	IFFP result;
	
	result = GetNextSubchunk(form, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		result = ReadGroup(&subchunk);
		if (result != IFF_NO_ERROR)
		{
			// either a PROP or Local chunk
			if (subchunk.ChunkID == 'PROP')
				return IFF_INVALID_CHUNK;
			else
			{
				result = ReadLocalChunk(&subchunk);
				if (result != IFF_NO_ERROR) return result;
			}
		}

		result = GetNextSubchunk(form, &subchunk);
	}
	
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP BFileIFF :: ReadLIST(IFFChunk *list)
{
	IFFChunk subchunk;
	bool propok = true;
	IFFP result;
	
	result = GetNextSubchunk(list, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		result = ReadGroup(&subchunk);
		if (result != IFF_NO_ERROR)
		{
			// could be a PROP chunk
			if (subchunk.ChunkID == 'PROP')
			{
				if (propok)
				{
					propok = false;
					result = ReadPROP(&subchunk);
					if (result != IFF_NO_ERROR) return result;
				}
				else
					return IFF_INVALID_CHUNK;
			}
			else
				return IFF_INVALID_CHUNK;
		}

		result = GetNextSubchunk(list, &subchunk);
	}
	
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP BFileIFF :: ReadCAT(IFFChunk *cat)
{
	IFFChunk subchunk;
	IFFP result;
	
	result = GetNextSubchunk(cat, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		result = ReadGroup(&subchunk);
		if (result != IFF_NO_ERROR) return result;

		result = GetNextSubchunk(cat, &subchunk);
	}
	
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP BFileIFF :: ReadPROP(IFFChunk *prop)
{
	IFFChunk subchunk;
	IFFP result;
	
	result = GetNextSubchunk(prop, &subchunk);
	while (result == IFF_NO_ERROR)
	{
		if (subchunk.ChunkID == 'FORM' || subchunk.ChunkID == 'LIST'
				|| subchunk.ChunkID == 'CAT ' || subchunk.ChunkID == 'PROP')
			return false;
			
		if (!ReadPropertyChunk(&subchunk)) return false;

		result = GetNextSubchunk(prop, &subchunk);
	}
	
	if (result == IFF_END_OF_CHUNK)
		return IFF_NO_ERROR;
	else
		return result;
}

IFFP BFileIFF :: ReadLocalChunk(IFFChunk *local)
{
	return IFF_NO_ERROR;
}

IFFP BFileIFF :: ReadPropertyChunk(IFFChunk *property)
{
	return IFF_NO_ERROR;
}

IFFP BFileIFF :: ReadChunkHeader(IFFChunk *chunk)
{
	off_t p;

	// Read in id and size
	p = chunk->ChunkStart;
	if (ReadAt(p, &(chunk->ChunkID), 4) != 4) return IFF_INVALID_CHUNK;
	if (ReadAt(p + 4, &(chunk->ChunkSize), 4) != 4) return IFF_INVALID_CHUNK;
	
	return IFF_NO_ERROR;
}

IFFP BFileIFF :: ReadGroup(IFFChunk *group)
{
	IFFP status;
	
	switch(group->ChunkID)
	{
		case 'FORM':
			status = ReadFORM(group);
			break;
			
		case 'LIST':
			status = ReadLIST(group);
			break;
			
		case 'CAT ':
			status = ReadCAT(group);
			break;
			
		default:
			status = IFF_INVALID_GROUP;
	}
	
	return status;
}

IFFP BFileIFF :: GetNextSubchunk(IFFChunk *parent, IFFChunk *subchunk)
{
	IFFP status;
	
	// Are we at the start? If so find first subchunk
	if (parent->ChunkPosition == parent->ChunkStart)
		parent->ChunkPosition = parent->ChunkStart
			+ sizeof(parent->ChunkID) + sizeof(parent->ChunkSize) + sizeof(parent->ChunkID);
	
	// Are we at the end?
	if (parent->ChunkPosition == parent->ChunkStart
		+ sizeof(parent->ChunkID) + sizeof(parent->ChunkSize) + parent->ChunkSize)
		return IFF_END_OF_CHUNK;
		
	// Read chunk and calculate next chunk position
	subchunk->ChunkParent = parent;
	subchunk->ChunkStart = parent->ChunkPosition; 
	subchunk->ChunkPosition = parent->ChunkPosition;

	status = ReadChunkHeader(subchunk);
	if (status != IFF_NO_ERROR) return status;

	parent->ChunkPosition += sizeof(subchunk->ChunkID) + sizeof(subchunk->ChunkSize)
		+ subchunk->ChunkSize;
			
	return IFF_NO_ERROR;
}

IFFP BFileIFF :: ReadIFF()
{
	IFFChunk ifffile;
	IFFP status;
	
	ifffile.ChunkStart = 0;
	ifffile.ChunkPosition = 0;
	ifffile.ChunkParent = NULL;
	
	status = ReadChunkHeader(&ifffile);
	if (status != IFF_NO_ERROR) return status;
	return ReadGroup(&ifffile);
}

