/*
	FileIFF.h

	IFF EA 85 File object. Adapted from Jerry Morrison and Steve Shaw's
	public domain source, iff.h, iffr.c, iffw.c
	
	Leo Chan
	August 1997
*/


#ifndef FILEIFF_H
#define FILEIFF_H

#include <File.h>

typedef int IFFP;

#define IFF_NO_ERROR		0L		// operation successful
#define IFF_END_OF_CHUNK	-1L		// end of chunk; no more subchunks
#define IFF_INVALID_CHUNK	-2L		// invalid chunk encountered
#define IFF_INVALID_GROUP	-3L		// invalid group chunk encountered

class IFFChunk
{
	friend class BFileIFF;

	protected:
		unsigned long ChunkPosition;	// position of next subchunk
		
	public:
		long ChunkID;
		unsigned long ChunkSize;
		unsigned long ChunkStart;	// Logical position of chunk in file
		IFFChunk *ChunkParent;
};

class BFileIFF : public BFile
{
	protected:
		// returns IFF_NO_ERROR if group is a valid IFF group
		IFFP ReadGroup(IFFChunk *group);
		
		// returns IFF_NO_ERROR is chunk header is successfully read
		IFFP ReadChunkHeader(IFFChunk *chunk);
		
		// these 4 functions are called every time the respective group chunks are
		// encountered.  Returns true if the operation was successful
		virtual IFFP ReadFORM(IFFChunk *form);
		virtual IFFP ReadLIST(IFFChunk *list);
		virtual IFFP ReadCAT(IFFChunk *cat);
		virtual IFFP ReadPROP(IFFChunk *prop);
		
		// the following two functions are called every time a data chunk is encountered
		// and are used by the above four virtual funcions
		
		// !NOTE!: overriding the above virtual functions means you are responsible for
		// calling ReadLocalChunk and ReadPropertyChunk! It is not neccessary to call them.
		// They are included if the default behaviour of the above 4 virtual functions are
		// desirable and the behaviour when data chunks are encountered needs to be changed.
		
		// ReadLocalChunk is called for subchunks of a FORM chunk
		// ReadPropertyChunk is called for subchunks of a PROP chunk
		// the functions must return true if no errors were encountered
		virtual IFFP ReadLocalChunk(IFFChunk *local);
		virtual IFFP ReadPropertyChunk(IFFChunk *property);

		// gets the next subchunk of parent; returns IFF_NO_ERROR if successful
		IFFP GetNextSubchunk(IFFChunk *parent, IFFChunk *subchunk);	
			
	public:
		BFileIFF(const char *path, uint32 openMode) : BFile(path, openMode){};
		
	// returns IFF_NO_ERROR if valid IFF file
		IFFP ReadIFF();
};

#endif
